"""Functions related to comparisons, didn't want to name the file tcr_test.py to avoid confusion with testing as in testing if code works."""

from collections.abc import Callable
from typing import Any


def able(func: Callable, *args, able_exception__: BaseException = Exception, **kwargs):
  """### Return True if x is func(x)-able aka does not raise any errors while passed in to func otherwise return False.

  This does not return the result, only bool whether it didn't raise an error in the process.

  #### Example:
  ```py
  >>> able(int, '1') # Wouldn't raise any Exception
  True
  >>> able(int, 'a') # Would raise: ValueError: invalid literal for int() with base 10: 'a'
  False
  >>> [int(x) for x in [] if able(int, x)]
  ```
  """
  try:
    func(*args, **kwargs)
  except able_exception__:
    return False
  else:
    return True


def isdunder(s: str):
  """Return True if the string ends and starts with at least two underscores each.

  A 2-char string containing just underscores is considered dunder.
  """

  return s.startswith('__') and (s.endswith('__'))



import re as regex


def commafy(text: str | int, splitter: str = ','):
  """Commafy a number.

  Returns a string of that number split by commas if needed, or any character passed as splitter.
  """
  if isinstance(text, int) and text < 0:
    return '-' + commafy(-text)
  text = str(text)
  temp = ''
  for i, letter in enumerate(text[::-1]):
    temp += letter
    if i % 3 == 2 and i != len(text) - 1:
      temp += splitter
  return temp[::-1]

def join_urlstr(*urlstrs: str) -> str:
  """Join a sequence of urlstrs with '/'. Cleans up extra slashes."""
  s = [str(x).strip('/') for x in urlstrs]
  s = '/'.join(s)
  return regex.sub(r'/+', '/', s)



from builtins import dir as __dir

dir = __dir


def dir2(x: object, /) -> list[str]:
  return [y for y in dir(x) if not y.startswith('__')]


def dir3(x: object, /) -> list[str]:
  return [y for y in dir(x) if not y.startswith('_')]



from ..src.tcr_print import fmt_iterable


class Snowflake(int):
  """Snowflake is an integer in `range((1 << 64) - 1)`."""

  def __tcr_display__(self, **kwargs):
    return fmt_iterable(int(self), **kwargs)


def is_snowflake(snowflake: Snowflake | int | str, *, allow_string=True) -> bool:
  if not isinstance(snowflake, ((int | str) if allow_string else int)):
    return False

  if not str(snowflake).isnumeric() or (int(snowflake) not in range((1 << 64) - 1)):
    return False

  return True
